Icybee's epic demonstration
===========================

You have downloaded Icybee's epic demonstration and I would like to thank you.

With this archive you can install the demonstration of the CMS on your
computer or server and obtain something like <http://icybee.org>. The
archive contains files to copy, SQL data to import and plenty to discover.

Icybee is a CMS for PHP5.3.6+, brand new and wonderful, built on solid
components, fresh ideas and years of experience from his dad.

This README will allows you to check the prerequisites of the CMS, will guide
you through its install and, if necessary, will help you to resolve issues.




## Prerequisites

To function properly Icybee requires PHP 5.3.6+ et MySQL 5.1+. The extension
PDO must be activated with the drivers for MySQL et SQLite (yes we are using
two database). URL rewriting must also be activated in Apache.

These prerequisites are fairly standard in a Linux install.




## Installation

The installation consists of four steps:

1. Copy the files in the archive
2. Ensure that PHP has the necessary permissions
3. Import SQL data
4. Configure the connection to the database 



### Copy the files in the archive

In the archive, the file "icybee-demo-YYYYMMDD" (e.g. icybee-demo-19770606)
contains all data files and scripts that make up the demo. The first step
is to copy these files to the root of your server, or the root directory of
your virtual host.

Caution! Some critical files may not appear on your operating system, such as
the ".htaccess" files used to locally configure Apache. Make sure that the file
".htaccess" is at the root of the destination directory once copying is
complete.

The simplest way might be to copy the directory itself then rename it.




### Make sure that the directory "repository" is writable by PHP

The directory "repository" is the working directory of Icybee. It contains
the files you upload from the CMS, the thumbnails of your images, caches
of the [ICanBoogie](http://icanboogie.org/) framework or the CMS,
persistent variables and many other things. It is imperative that PHP has
write access on the directory to be able to change the content.

Under Linux or MacOS, type "make setup" from the terminal :

	$ make setup

Or the following line, by replacing "/path/to/repository" by the path to the
directory "repository" :

	$ sudo chown www-data:www-data -R /path/to/repository
	$ sudo chmod -R 705 /path/to/repository

Under Windows you have nothing to do if the destination directory is
usually modifiable without UAC popping in your face.




### Import SQL data

The file "icybee-demo-YYYYMMDD.sql" (.e.g. icybee-demo-19770606) contains an
SQL export that must be imported into a database for the demo to work.

The demonstration is configured to use the database "icybee-demo". You can
either create a databse named "icybee-demo" or import the data into a SQL
database that already exists. The tables are prefixed with `icybeedemo__`, so
you can import the SQL data without fear.

Caution! It is imperative that the character set of the database tables is
"utf8_general_ci". 




### Configure the connection to the databse

The demonstration is configured to connect to the database "icybee-demo"
with the user "root" and the password "root", and to use the prefix
`icybeedemo__` for its tables.

To configure the connection, edit the file
"protected/all/config/core.php" :

	'connections' => array
	(
		'primary' => array
		(
			'dsn' => 'mysql:dbname=icybee-demo',
			'username' => 'root',
			'password' => 'root',
			'timezone' => '+01.00',
			'options' => array
			(
				'#prefix' => 'icybeedemo_'
			)
		)
	)

The `dsn`, `username`, `password` and `#prefix` keys should allow you to
configure the connection to the database as required.

Attention to the value of `#prefix`! An underscore is added to the prefix if
it is defined. This is not `icybeedemo_` that will be used but `icybeedemo__`.
(We use two underscore so that PMA creates a group).




## Connect to the administration interface of Icybee

The administration interface of Icybee is accessible from the path
"/admin/". This path is relative to the site you want to administer. The
demonstration uses three sites, one for each language. The following paths
allow you to reach the administration interface of each version:

- "/admin/"
- "/fr/admin/"
- "/es/admin/"

Of course, it is possible to switch from one version to the other from the
administration interface.

Two accounts exist to administer the websites : one with full powers, and
another with limited powers.




### The full powers account

Identifier: `admin`  
Password: `admin123`




### The limited powers account

Identifier: `user`  
Password: `user123`




## Known problems

There is no date-picker because the one I was using is no longer compatible
with the latests versions of MooTools. Because of that, "Date" fields display
the date in the international format "YYYY-MM-DD HH-MM-SS".




## Troubleshooting

Despite all my work, the software might crash for a foul and mysterious reason.
Before reporting the problem you can try the following:




### Check the prerequisites

- Check the minimum version of the software required.

- Check that the directory "repository" is writable by PHP.

- Enable the Apache URL rewriting module:

	```
	$ sudo a2enmod rewrite
	$ sudo /etc/init.d/apache2 restart




### Is the file ".htaccess" at the root of the website ?

Check that the file `.htaccess` is at the root of the website. This file
might be hidden on some operation system. If you copied the files it is
possible that it was overlooked.




### Disable caches

The framework [ICanBoogie](http://icanboogie.org/) uses files to cache
parameters that are expensive to create. You should disable these caches
before trying to locate the source of the problem.

Edit the configuration file "protected/all/config/core.php" and remove the
comment on the following lines:

	// 	'cache assets' => false,
	// 	'cache bootstrap' => false,
	// 	'cache catalogs' => false,
	// 	'cache configs' => false,
	// 	'cache modules' => false,

They will override the configuration parameters of Icybee and disable most caches.




### Remove .phar files

The framework [ICanBoogie](http://icanboogie.org/) and the toolkit
[Brickrouge](http://brickrouge.org/) can be used as Phar. The demonstration
don't used them by default but you might create them to test the software.
In which case you should delete the files "Icybee/framework/ICanBoogie.phar"
and "Icybee/framework/Brickrouge.phar".




## Report a problem

If despite your valiant efforts the problem persists please open a ticket on
GitHub:

<https://github.com/Icybee/Icybee/issues>

Don't forget to specify your server configuration including the version of
PHP and MySQL, but also your browser and operating system. 




## A last word

The demonstration only highlights some of the many features of Icybee. Its
purpose is to present the administration interface of the CMS, its work-flow,
how it manages multiple websites, pages, articles and their organization.
Hoping to make you want to find out more and maybe join the project.

Feel free to send me your comments are remarks. Constructive critics are
always welcome, even if they flay a little my sensitive developer heart :)

I wish you a happy discovery. Have fun and see you soon!

Olivier, [@olvlv](https://twitter.com/olvlvl)