La démonstration épique d'Icybee
================================

Vous avez téléchargé la démonstration épique d'Icybee et je vous en remercie.

Cette archive vous permettra d'installer la démonstration épique du CMS Icybee
chez vous ou sur un serveur. L'archive contient des fichiers à copier, des
données SQL à importer et plein de choses à découvrir.

Icybee est un CMS pour PHP5.3.6+, tout neuf et tout merveilleux, bâti sur des
composants solides, des idées fraiches et les années d'expérience de son papa.

Ce README vous permettra de vérifier les pré-requis du CMS, vous guidera
durant son installation et, si cela est nécessaire, vous aidera à la
résolution de problèmes.



## Pré-requis

Pour fonctionner correctement Icybee requiert la version 5.3.6+ de PHP et la
version 5.1+ de MySQL. L'extension PDO doit être activée avec les drivers
MySQL et SQLite (et oui, on utilise deux bases de données). La réécriture
d'URL d'Apache doit également être activée.

Des pré-requis assez bateau pour une installation linux classique.




## Installation

L'installation se déroule en quatre étapes :

1. Copier les fichiers de l'archive
2. S'assurer que PHP dispose des permissions nécessaires
3. Importer les données SQL
4. Configurer la connexion à la base de données




### Copier les fichiers de l'archive

Dans l'archive, le dossier "icybee-demo-YYYYMMDD" (e.g. icybee-demo-19770606)
contient l'ensemble des fichiers de données et de scripts qui composent la
démo. La première étape consiste à copier ces fichiers à la racine de votre
serveur, ou à la racine du dossier d'un hôte virtuel.

Attention ! Certains fichiers essentiels peuvent ne pas apparaitre sur votre
système d'exploitation, comme les fichiers ".htaccess" utilisés pour configurer
localement le serveur Apache. Assurez-vous que le fichier ".htaccess" est bien
présent à la racine du dossier de destination une fois la copie terminée.

Le plus simple sera peut-être de copier le dossier lui-même puis de le
renommer.




### S'assurer que le dossier "repository" est modifiable par PHP

Le dossier "repository" est le dossier de travail d'Icybee. Il contient les
fichiers que vous uploadez depuis le CMS, les miniatures des images, les
caches du framework ICanBoogie ou du CMS, les variables persistantes et
plein d'autres choses. Il est impératif que PHP ait un droit d'écriture sur
ce dossier afin d'être en mesure d'en modifier le contenu.

Sous Linux ou MacOS, saisissez "make setup" depuis le terminal :

	$ make setup

Ou les lignes suivantes, en remplaçant "/path/to/repository" par le chemin
vers le dossier "repository" :

	$ sudo chown www-data:www-data -R /path/to/repository
	$ sudo chmod -R 705 /path/to/repository

Sous Windows vous n'avez rien à faire si le dossier de destination est
habituellement modifiable sans que l'UAC ne vous saute à la gorge.




### Importer les données SQL

Le fichier "icybee-demo-YYYYMMDD.sql" (.e.g icybee-demo-19770606) contient un
export SQL des données de la démonstration qui doit être importé dans une base
de données pour que la démonstration fonctionne.

La démonstration est configurée pour utiliser la base de données "icybee-demo".
Vous pouvez soit créer une base de données nommée "icybee-demo", soit
importer les données SQL dans un base de données déjà existante. Les tables
sont préfixées par `icybeedemo__`, vous pouvez donc importer les données SQL
sans craintes.

Attention ! Il est impératif que le jeu de caractères des tables de la base de
données soit "utf8_general_ci".




### Configurer la connexion à la base de données

La démonstration est configurée pour se connecter à la base de données
"icybee-demo" avec l'utilisateur "root" et le mot de passe "root", et pour
utiliser le préfixe `icybeedemo__` pour ses tables.

Pour configurer la connection, éditez le fichier
"protected/all/config/core.php" :

	'connections' => array
	(
		'primary' => array
		(
			'dsn' => 'mysql:dbname=icybee-demo',
			'username' => 'root',
			'password' => 'root',
			'timezone' => '+01.00',
			'options' => array
			(
				'#prefix' => 'icybeedemo_'
			)
		)
	)

Les clés `dsn`, `username`, `password` et `#prefix` devraient vous permettre de
configurer la connection à la base de données selon vos besoins.

Attention à la valeur de `#prefix` ! Un underscore sera ajouté au préfixe s'il
est défini. Ce n'est donc pas `icybeedemo_` qui sera utilisé comme préfixe
mais bien `icybeedemo__`. (On utilise ici deux underscores pour que PMA crée un
groupe.)




## Se connecter à l'interface d'administration d'Icybee

L'interface d'administration d'Icybee est accessible depuis le chemin
"/admin/". Ce chemin est relatif au site que vous souhaitez administrer. La
démonstration épique d'Icybee utilise trois sites, un pour chaque version de
langue. Les chemins suivants vous permettent d’atteindre l'interface
d'administration de chaque version :

- "/admin/"
- "/fr/admin/"
- "/es/admin/"

Bien sûr, il est possible de passer d'une version à l'autre depuis l'interface
d'administration.

Deux comptes existent pour administrer les sites : un compte
"plein pouvoirs" et un compte restreint.




### Un compte "plein pouvoirs"

Identifiant: admin  
Mot de passe: admin123




### Un compte restreint

Identifiant: user  
Mot de passe: user123




## Problèmes connus

Il n'y a pas de sélecteur de date parce que celui que j'utilisais n'est plus
compatible avec les dernières versions de MooTools. Pour cette raison les champs
"Date" affichent une date au format international "YYYY-MM-DD HH-MM-SS".




## Résolution de problèmes

Malgré tout mon travail, il se peut que pour une raison infecte et mystérieuse
le logiciel plante. Avant de signaler le problème vous pouvez tenter les
choses suivantes :




### Vérifier les pré-requis

- Vérifiez la version minimum des logiciels.

- Vérifier que le dossier "repository" est modifiable par PHP.

- Activez le module de réécriture d'URL d'Apache :

	```
	$ sudo a2enmod rewrite
	$ sudo /etc/init.d/apache2 restart




### Le fichier ".htaccess" se trouve-t-il bien à la racine du site ?

Vérifiez que le fichier `.htaccess` se trouve bien à la racine du site. Ce
fichier est caché sur certains systèmes d'exploitation. Si vous avez copié les
fichiers de la démonstration il est possible qu'il ait été oublié.




### Désactiver les caches

Le framework [ICanBoogie](http://icanboogie.org/) utilise des fichiers pour
mettre en cache des paramètres couteux à constituer. Désactivez ses caches
avant de continuer à chercher la cause du problème.

Pour cela éditez le fichier de configuration "protected/all/config/core.php",
et décomentez les lignes suivantes :

	// 	'cache assets' => false,
	// 	'cache bootstrap' => false,
	// 	'cache catalogs' => false,
	// 	'cache configs' => false,
	// 	'cache modules' => false,

Ces lignes permettent de supplanter les paramètres de configuration d'Icybee
et ainsi de désactiver certains caches.




### Supprimer les fichiers .phar

Le framework [ICanBoogie](http://icanboogie.org/) et le toolkit
[Brickrouge](http://brickrouge.org) peuvent être utilisés en tant que Phar. La
démonstration ne les utilise pas sous cette forme par défaut, mais vous pouvez
créer ces fichiers pour tester. Supprimez les fichiers
"Icybee/framework/ICanBoogie.phar" et "Icybee/framework/Brickrouge.phar" en
cas de problèmes.




## Signaler un problème

Si malgré vos valeureux efforts le problème persiste n'hésitez pas à ouvrir un
billet sur GitHub :

<https://github.com/Icybee/Icybee/issues>

N'oubliez pas de préciser la configuration de votre serveur et notamment les
versions des logiciels PHP et MySQL, mais aussi de votre navigateur et système
d'exploitation.




## Un dernier mot

La démo ne met en lumière que quelques unes des nombreuses fonctionnalités
d'Icybee. Son but est de présenter l'interface d'administration du CMS, son
flux de travail, la gestion du multi-sites, des pages, des articles et de leur
organisation. En espérant vous donner envie d'en découvrir davantage et
pourquoi pas de vous joindre au projet.

N'hésitez pas à me faire part de vos commentaires et remarques. Les critiques
constructives sont toujours les bienvenues, même si elles écorchent un peu mon
cœur sensible de développeur :)

Je vous souhaite une bonne découverte. Amusez-vous bien et à bientôt !

Olivier, [@olvlv](https://twitter.com/olvlvl)